unit STestComponent;

interface
              
uses              
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, OpenWirePins, OpenWireBasicStreamTypes, ExtCtrls, StdCtrls, ComCtrls;

type TPinNotificationEvent = function (Handler : IOWStream; Operation : Integer; Data : Pointer) : TOWNotificationResults of object;
type TUpdateEvent = procedure ( Data: Integer ) of object;
type TSendFloatEvent = procedure ( Data: Single ) of object;

type              
  ITestStream = interface(IOWStream)
    ['{C19C2C81-5E66-41D4-B365-3A8DB22D67A2}']
    procedure UpdateData ( Data : Integer ); stdcall;
  end;

//  TSBaseTestComponent = class;

  TTestSourcePin = class(TOWSourcePin)

  protected
//    FComponent : TSBaseTestComponent;
    UpdateEvent                 : TUpdateEvent;
    PinNotificationEvent        : TPinNotificationEvent;

  public
    constructor Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; APinNotificationEvent : TPinNotificationEvent );

  protected
  public
    function Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults; virtual;
  public
  end;

  TTestSinkPin = class(TOWSinkPin, ITestStream )

  protected
    UpdateEvent                 : TUpdateEvent;
    PinNotificationEvent        : TPinNotificationEvent;
//    FComponent : TSBaseTestComponent;

  public
    procedure UpdateData ( Data : Integer ); stdcall;

  protected
  public
    constructor Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; APinNotificationEvent : TPinNotificationEvent );

  public
  end;

  TTestComboSinkPin = class( TTestSinkPin, IOWFloatStream )

  protected

    SendFloatEvent : TSendFloatEvent;
    
  public
    procedure SendFloatData ( Data : Single ); stdcall;

  protected
  public
    constructor Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; ASendFloatEvent : TSendFloatEvent; APinNotificationEvent : TPinNotificationEvent );
    
  end;

  TSTestComponent = class(TComponent)
  private
    { Private declarations }

  protected
    { Protected declarations }
    FInput  : TTestSinkPin;
    FOutput : TTestSourcePin;
    FInverseInput  : TTestSinkPin;
    FInverseOutput : TTestSourcePin;

//    FCollection : TOWPinList;
    FExtendedOutputs : TOWPinList;
    FExtendedInputs  : TOWPinList;
    FCombo           : TOWPinList;

  protected
    { Protected declarations }
    function PinNotification( Handler : IOWStream; Operation : Integer; Data : Pointer ) : TOWNotificationResults;
    procedure UpdateData ( Data : Integer );

  protected
    function  GetExtendedOutputs() : Integer;
    procedure SetExtendedOutputs( Value : Integer );

    function  GetExtendedInputs() : Integer;
    procedure SetExtendedInputs( Value : Integer );
    
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    { Published declarations }
    property Input  : TTestSinkPin read FInput  write FInput;
    property Output : TTestSourcePin   read FOutput write FOutput;
    property InverseInput  : TTestSinkPin read FInverseInput  write FInverseInput;
    property InverseOutput : TTestSourcePin   read FInverseOutput write FInverseOutput;
    property Combo : TOWPinList read FCombo write FCombo;
    property ExtendedOutputs : TOWPinList read FExtendedOutputs write FExtendedOutputs;
    property ExtendedInputs : TOWPinList read FExtendedInputs write FExtendedInputs;
    property CountExtendedOutputs : Integer read GetExtendedOutputs write SetExtendedOutputs;
    property CountExtendedInputs : Integer read GetExtendedInputs write SetExtendedInputs;
  end;

  TSTestClockComponent = class(TTimer)
  private
    { Private declarations }
    
  protected
    { Protected declarations }
    FOutput : TTestSourcePin;
    FCounter : Integer;
    
  protected
    procedure Timer; override;
    procedure Loaded; override;

  protected
    procedure STestClockComponentTimer(Sender: TObject);
    
  protected
    { Protected declarations }
    function PinNotification( Handler : IOWStream; Operation : Integer; Data : Pointer ) : TOWNotificationResults;
    
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    { Published declarations }
    property Output : TTestSourcePin   read FOutput write FOutput;
  end;

  TSTestProgressComponent = class(TProgressBar)
  private
    { Private declarations }
    
  protected
    { Protected declarations }
    FInput   : TTestSinkPin;

  protected
    { Protected declarations }
    procedure UpdateData ( Data : Integer );
    
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    { Published declarations }
    property Input : TTestSinkPin   read FInput write FInput;
  end;

  TSTestLabelComponent = class(TLabel)
  private
    { Private declarations }
    
  protected
    { Protected declarations }
    FInput   : TTestComboSinkPin;

  protected
    { Protected declarations }
    procedure UpdateData ( Data : Integer );
    procedure SendData ( Data : Single );

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    { Published declarations }
    property Input : TTestComboSinkPin   read FInput write FInput;
  end;

procedure Register;

implementation
constructor TSTestComponent.Create(AOwner: TComponent);
begin
  inherited;
  FInput  := TTestSinkPin.Create( Self, UpdateData, PinNotification );
  FOutput := TTestSourcePin.Create( Self, UpdateData, PinNotification );
  FOutput.AddFunctionSource( FInput );

  FInverseInput  := TTestSinkPin.Create( Self, UpdateData, PinNotification );
  FInverseOutput := TTestSourcePin.Create( Self, UpdateData, PinNotification );
  FInverseOutput.AddFunctionSource( FInput );

  FCombo := TOWPinList.Create( False );
  FCombo.AddNamed( TTestSinkPin.Create( Self, UpdateData, PinNotification ), 'Test Input');
  FCombo.AddNamed( TTestSourcePin.Create( Self, UpdateData, PinNotification ), 'Test Output');


  FExtendedOutputs := TOWPinList.Create( False );
  FExtendedInputs := TOWPinList.Create( False );
end;


destructor TSTestComponent.Destroy;
var
  I : Integer;
  
begin
  for I := FExtendedOutputs.Count - 1 downto 0 do
    FExtendedOutputs.Pins[ I ].Free();

  FExtendedOutputs.Free;

  for I := FExtendedInputs.Count - 1 downto 0 do
    FExtendedInputs.Pins[ I ].Free();

  FExtendedInputs.Free;
  
  for I := FCombo.Count - 1 downto 0 do
    FCombo.Pins[ I ].Free();

  FCombo.Free;

  FOutput.Free;
  FInput.Free;
  FInverseOutput.Free;
  FInverseInput.Free;
  inherited;
end;
//---------------------------------------------------------------------------
function  TSTestComponent.GetExtendedOutputs() : Integer;
begin
  Result := FExtendedOutputs.Count;
end;
//---------------------------------------------------------------------------
procedure TSTestComponent.SetExtendedOutputs( Value : Integer );
var
  Pin : TOWPin;
begin
  while( FExtendedOutputs.Count > Value ) do
    begin
    Pin := FExtendedOutputs.Pins[ FExtendedOutputs.Count - 1 ];
    FExtendedOutputs.Remove( Pin );
    Pin.Free;
    end;

  while( FExtendedOutputs.Count < Value ) do
    begin
    FExtendedOutputs.Add( TTestSourcePin.Create( Self, UpdateData, PinNotification ));
    end;
end;
//---------------------------------------------------------------------------
function  TSTestComponent.GetExtendedInputs() : Integer;
begin
  Result := FExtendedInputs.Count;
end;
//---------------------------------------------------------------------------
procedure TSTestComponent.SetExtendedInputs( Value : Integer );
var
  Pin : TOWPin;

begin
  while( FExtendedInputs.Count > Value ) do
    begin
    Pin := FExtendedInputs.Pins[ ExtendedInputs.Count - 1 ];
    FExtendedInputs.Remove( Pin );
    Pin.Free;
    end;

  while( FExtendedInputs.Count < Value ) do
    begin
    FExtendedInputs.Add( TTestSinkPin.Create( Self, UpdateData, PinNotification ));
    end;
end;
//---------------------------------------------------------------------------
function TSTestComponent.PinNotification( Handler : IOWStream; Operation : Integer; Data : Pointer ) : TOWNotificationResults;
begin
  Result := [];
end;
//---------------------------------------------------------------------------
procedure TSTestComponent.UpdateData ( Data : Integer );
begin
end;
//---------------------------------------------------------------------------
constructor TSTestClockComponent.Create(AOwner: TComponent);
begin
  inherited;
  FOutput := TTestSourcePin.Create( Self, NIL, PinNotification );
  FCounter := 0;
end;
//---------------------------------------------------------------------------
destructor TSTestClockComponent.Destroy;
begin
  FOutput.Free;
  inherited;
end;

//---------------------------------------------------------------------------
function TSTestClockComponent.PinNotification( Handler : IOWStream; Operation : Integer; Data : Pointer ) : TOWNotificationResults;
var
  Interf : ITestStream;

begin
  if( Handler.QueryInterface(ITestStream,Interf) = 0 ) then
      Interf.UpdateData( Operation ); 

  Result := [];
end;
//---------------------------------------------------------------------------
procedure TSTestClockComponent.Timer;
begin
  if( csDesigning in ComponentState ) then
    FOutput.Notify( 0, NIL )
    
  else
    FOutput.Notify( FCounter, NIL );

  Inc( FCounter );
  if( FCounter > 100 ) then
    FCounter := 0;

  inherited Timer;
end;
//---------------------------------------------------------------------------
procedure TSTestClockComponent.Loaded;
begin
  if( ( not Assigned( OnTimer ) ) and ( not ( csDesigning in ComponentState ) )) then
    OnTimer := STestClockComponentTimer;

end;
//---------------------------------------------------------------------------
procedure TSTestClockComponent.STestClockComponentTimer(Sender: TObject);
begin
end;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
constructor TSTestProgressComponent.Create(AOwner: TComponent);
begin
  inherited;
  FInput := TTestSinkPin.Create( Self, UpdateData, NIL );
end;
//---------------------------------------------------------------------------
destructor TSTestProgressComponent.Destroy;
begin
  FInput.Free;
  inherited;
end;

//---------------------------------------------------------------------------
procedure TSTestProgressComponent.UpdateData ( Data : Integer );
begin
  Position := Data; 
end;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
constructor TSTestLabelComponent.Create(AOwner: TComponent);
begin
  inherited;
  FInput := TTestComboSinkPin.Create( Self, UpdateData, SendData, NIL );
end;
//---------------------------------------------------------------------------
destructor TSTestLabelComponent.Destroy;
begin
  FInput.Free;
  inherited;
end;

//---------------------------------------------------------------------------
procedure TSTestLabelComponent.UpdateData ( Data : Integer );
begin
  Caption := IntToStr( Data ); 
end;
//---------------------------------------------------------------------------
procedure TSTestLabelComponent.SendData ( Data : Single );
begin
  Caption := FloatToStr( Data ); 
end;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
constructor TTestSourcePin.Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; APinNotificationEvent : TPinNotificationEvent );
begin
  inherited Create( AOwner );
  UpdateEvent := AUpdateEvent;
  PinNotificationEvent := APinNotificationEvent;

  AddType( ITestStream, Notification );
end;
//---------------------------------------------------------------------------
constructor TTestSinkPin.Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; APinNotificationEvent : TPinNotificationEvent );
begin
  inherited Create( AOwner );
  UpdateEvent := AUpdateEvent;
  PinNotificationEvent := APinNotificationEvent;
end;
//---------------------------------------------------------------------------
function TTestSourcePin.Notification( Handler : IOWStream; Operation : Integer; Data : Pointer; State : TOWNotificationState ) : TOWNotificationResults;
begin
  if( Assigned( PinNotificationEvent )) then
    Result := PinNotificationEvent( Handler, Operation, Data )

  else
    Result := [];
    
end;
//---------------------------------------------------------------------------
procedure TTestSinkPin.UpdateData ( Data : Integer );
begin
  if ( Assigned( UpdateEvent )) then
    UpdateEvent( Data );
end;
//---------------------------------------------------------------------------
constructor TTestComboSinkPin.Create( AOwner: TComponent; AUpdateEvent : TUpdateEvent; ASendFloatEvent : TSendFloatEvent; APinNotificationEvent : TPinNotificationEvent );
begin
  inherited Create( AOwner, AUpdateEvent, APinNotificationEvent );
  SendFloatEvent := ASendFloatEvent;
end;
//---------------------------------------------------------------------------
procedure TTestComboSinkPin.SendFloatData ( Data : Single );
begin
  if ( Assigned( SendFloatEvent )) then
    SendFloatEvent( Data );
end;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
{
type TTestIntegerPropertyEditor = class(TPropertyEditor)
  public
    function  GetAttributes: TPropertyAttributes; override;
    function  GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;
//---------------------------------------------------------------------------
function  TTestIntegerPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [ paRevertable, paMultiSelect ];
end;
//---------------------------------------------------------------------------
function  TTestIntegerPropertyEditor.GetValue: string;
begin
  Result := IntToStr( Integer( GetOrdValue()));
end;
//---------------------------------------------------------------------------
procedure TTestIntegerPropertyEditor.SetValue(const Value: string);
var
//  Component : TPersistent;
  SelectionList : TDesignerSelectionList;

begin
  SetOrdValue( StrToIntDef( Value, 0 ));
  if( Assigned( Designer)) then
    begin
    SelectionList := TDesignerSelectionList.Create;
    Designer.GetSelections( SelectionList );
//    SelectionList.Add( SelectionList.Items[ 0 ] );
//    Component := GetComponent( 0 );
//    Designer.SelectComponent( NIL );
    
    Designer.SetSelections( SelectionList );
//    Designer.SelectComponent( Component );
    SelectionList.Free();
    end;


end;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('OpenWire', [TSTestComponent]);
  RegisterComponents('OpenWire', [TSTestClockComponent]);
  RegisterComponents('OpenWire', [TSTestLabelComponent]);
  RegisterComponents('OpenWire', [TSTestProgressComponent]);
  OWRegisterWireStream( ITestStream, 'Demo Integer single' ); 
//  RegisterPropertyEditor( typeinfo(Integer), TSTestComponent, 'CountExtendedOutputs', TTestIntegerPropertyEditor );
end;

end.
